@@ -1,5 +1,6 @@
 import boto3
 from boto3.dynamodb.conditions import Attr, Key
+from botocore.exceptions import ClientError
 
 from fdbk import DBConnection
 from fdbk.utils import (
@@ -29,34 +30,55 @@ class DynamoDbConnection(DBConnection):
         self._topics_table.table_status
         self._data_table.table_status
 
-    def add_topic(self, name, **kwargs):
+    def add_topic(self, name, overwrite=False, **kwargs):
         topic_d = generate_topic_dict(name, add_id=True, **kwargs)
-        self._topics_table.put_item(
-            Item=topic_d,
-            ConditionExpression=Attr('id').not_exists(),
-        )
+        self.validate_template(topic_d)
+
+        params = {}
+        if not overwrite:
+            params["ConditionExpression"] = Attr('id').not_exists()
+
+        try:
+            self._topics_table.put_item(Item=topic_d, **params)
+        except ClientError:
+            raise KeyError(duplicate_topic_id(topic_d["id"]))
 
         return topic_d["id"]
 
-    def add_data(self, topic_id, values):
+    def add_data(self, topic_id, values, overwrite=False):
         topic_d = self.get_topic(topic_id)
         fields = topic_d["fields"]
+
+        if topic_d.get('type') != 'topic':
+            raise AssertionError('Can not add data to template.')
+
         values = obj_numbers_to_decimals(values)
+        params = {}
+        if not overwrite:
+            params["ConditionExpression"] = Attr(
+                'topic_id').not_exists() | Attr('timestamp').not_exists()
 
         data = generate_data_entry(
             topic_id, fields, values, convert_timestamps=True)
-        self._data_table.put_item(Item=data, ConditionExpression=Attr(
-            'topic_id').not_exists() | Attr('timestamp').not_exists(), )
+        try:
+            self._data_table.put_item(Item=data, **params)
+        except ClientError:
+            raise AssertionError(duplicate_topic_id(topic_d["id"]))
 
-    def get_topics(self, type_=None):
+        return data["timestamp"]
+
+    def get_topics_without_templates(self, type_=None, template=None):
         if type_:
             topics = self._topics_table.scan(
                 FilterExpression=Attr("type").eq(type_)).get('Items')
+        elif template:
+            topics = self._topics_table.scan(
+                FilterExpression=Attr("template").eq(template)).get('Items')
         else:
             topics = self._topics_table.scan().get('Items')
         return generate_topics_list(topics)
 
-    def get_topic(self, topic_id):
+    def get_topic_without_templates(self, topic_id):
         topic = self._topics_table.get_item(
             Key=dict(id=topic_id)
         ).get('Item')
